//Program to dump text using substrings.
// FIXED LENGTH .tbl entry  $xx=FIXED,BBBBB,C
//    where xx = hex value in script used to call substring table.
//    FIXED identifies fixed length type substrings.
//    BBBBB is the ROM offset of entry 0 in the substring table.
//    C is the substring length.
//    Enter all values in hexadecimal
//
//
// SNES .tbl entry   $xx=SNES,A,BBBBB,C,D
//    where xx = hex value in script used to call substring table.
//    A = 'L' for LoROM, 'H' for HiROM
//    BBBBB = Rom offset of substring pointer table
//    C = 2 for 2-byte pointers, 3 for 3 byte pointers
//    D = number of bytes in index
//

#include <cstdlib>
#include <iostream>

using std::cout;
using std::cin;
using std::hex;

int hex_return(char readbyte);
int check_str(unsigned char string1[], const char string2[]);
int hex_to_int(unsigned char string[]);
int power(int x, int y);

char romfile[260];
char tablefile[260];
char dumpfile[260];

int main()
{
    int x;
    int load_font_table(unsigned int font[], unsigned long int string[], unsigned char format[][4]);
    unsigned int font[0x10000];      //font table offsets (2bytes x 65536 chars)
    unsigned long int string[0x10000];    //string offsets  (4bytes x 65536 chars)
    unsigned char format[0x10000][4];  //format of string, per char, NEED MAKE IT PER CHAR
    unsigned long int rom_off=0x0;    //offset in ROM
    unsigned long int rom_end=0x0;    //last offset to read
    unsigned long int ptr_off=0;
    unsigned long int sub_off=0;          //substring offset
    unsigned long int write_off=0;        //offset in text dump
    unsigned long int entry;  //entry in font table, entry within current substring table
    unsigned int tbl_off;                 //offset in table file to read
    unsigned char readbyte;               //temp. value for reading a byte from ROM, etc.
    unsigned int str_entry;         //entry in string table
    unsigned int sub_length;        //max. substring length
    unsigned int ent_width;
    unsigned char hexstring[10];
    unsigned char font_loaded=0;
    unsigned int headersize;
    unsigned int romsize;
    int check_EOF;
    for(x=0;x<0x10000;x++)  //initalize font and string tables.
    {
        font[x]=0;
        string[x]=0xFFFB;   //undefined
    }
    cout<<"Light Text. v0.03 by KingMike\n";
    cout<<"ROM FILE NAME :";
    cin>>romfile;
    cout<<"TABLE FILE NAME :";
    cin>>tablefile;
    cout<<"DUMP TO FILE :";
    cin>>dumpfile;
    cout<<"START OFFSET :";
    cin>>hexstring;
    rom_off=hex_to_int(hexstring);
    cout<<"END OFFSET :";
    cin>>hexstring;
    rom_end=hex_to_int(hexstring);
    FILE *rom;
    if((rom=fopen(romfile,"rb"))==NULL)
    {
        cout<<"Can't open the ROM file \""<<romfile<<"\".\n";
        system("PAUSE");
        return EXIT_FAILURE;
    }
    FILE *tdump;
    if((tdump=fopen(dumpfile,"wb"))==NULL)
    {
        cout<<"Can't open the table file \""<<dumpfile<<"\".\n";
        system("PAUSE");
        return EXIT_FAILURE;
    }
    FILE *table;
    if((table=fopen(tablefile,"rb"))==NULL)
    {
        cout<<"Can't open the table file \""<<tablefile<<"\".\n";
        system("PAUSE");
        return EXIT_FAILURE;
    }
    if(load_font_table(font, string,format)==EXIT_FAILURE)
        return EXIT_FAILURE;
    fseek(rom,0,SEEK_END);
    romsize=ftell(rom);
    headersize=romsize%2048;
    while(rom_off<=rom_end)
    {
        fseek(rom,rom_off,SEEK_SET);
        readbyte=fgetc(rom);
        entry=readbyte;
        tbl_off=font[entry];
        if(tbl_off>0xFFF0)  //control codes
        {
            if(tbl_off==0xFFFF)   //line break
            {
                fseek(tdump,write_off,SEEK_SET);
                fputc((char)0x0D,tdump);
                fputc((char)0x0A,tdump);
                write_off+=2;
            }
            else if(tbl_off==0xFFFD)
            {
                if(font_loaded!=0)  //clear font loaded at End of string
                {
                    x=0;
                    while(tablefile[x]!='.')
                        x++;
                    x--;
                    tablefile[x]=0x30;
                    fclose(table);
                    if((table=fopen(tablefile,"rb"))==NULL)
                    {
                        cout<<"Can't open the table file \""<<tablefile<<"\".\n";
                        system("PAUSE");
                        return EXIT_FAILURE;
                    }
                    if(load_font_table(font,string,format)==EXIT_FAILURE)
                        return EXIT_FAILURE;
                    font_loaded=0;
                }
                fseek(tdump,write_off,SEEK_SET);
                fwrite("[END]",5,1,tdump);
                write_off+=5;
                fseek(tdump,write_off,SEEK_SET);
                fputc((char)0x0D,tdump);
                fputc((char)0x0A,tdump);
                write_off+=2;
            }
            else if(tbl_off==0xFFFE)
            {
                fseek(tdump,write_off,SEEK_SET);
                fwrite("[WAIT]",6,1,tdump);
                write_off+=6;
                fseek(tdump,write_off,SEEK_SET);
                fputc((char)0x0D,tdump);
                fputc((char)0x0A,tdump);
                write_off+=2;
            }
            else if(tbl_off==0xFFFC)  //substring code goes here
            {
                str_entry=entry;
                if(format[str_entry][0]==0)    //FIXED WIDTH SUBSTRINGS
                {
                    //get substring #
                    for(x=0;x<format[str_entry][2];x++)
                    {
                        rom_off++;
                        fseek(rom,rom_off,SEEK_SET);
                        readbyte=fgetc(rom);
                        entry=entry+readbyte*(power(256,x));
                    }
                    sub_off=string[entry];
                    sub_off=sub_off+(entry*format[str_entry][1]);
                    sub_length=format[str_entry][1];
                    //we have the ROM offset of the substring found.
                    fseek(rom,sub_off,SEEK_SET);
                    readbyte=fgetc(rom);
                    entry=readbyte;
                    tbl_off=font[entry];//check if the byte = EOL
                    while(tbl_off!=0xFFFD && sub_length>0)
                    {
                        fseek(table,tbl_off,SEEK_SET);
                        check_EOF=fgetc(table);
                        fseek(table,tbl_off,SEEK_SET);
                        readbyte=fgetc(table);
                        while(readbyte!=0x0D && readbyte!=0x0A && check_EOF!=EOF)
                        {
                            tbl_off++;
                            fseek(tdump,write_off,SEEK_SET);
                            fputc(readbyte,tdump);
                            write_off++;
                            fseek(table,tbl_off,SEEK_SET);
                            check_EOF=fgetc(table);
                            fseek(table,tbl_off,SEEK_SET);
                            readbyte=fgetc(table);
                        }
                        sub_off++;
                        sub_length--;
                        fseek(rom,sub_off,SEEK_SET);
                        readbyte=fgetc(rom);
                        entry=readbyte;
                        tbl_off=font[entry];
                    }
                }//end of fixed length format
                else if(format[str_entry][0]==2)    //SNES pointer format
                {
                     //get substring #
                    int index=0;
                    for(x=0;x<format[str_entry][3];x++)
                    {
                        rom_off++;
                        fseek(rom,rom_off,SEEK_SET);
                        readbyte=fgetc(rom);
                        if(x>1)
                        {
                            index=index+readbyte*(power(256,x));
                        }
                        else
                            index=readbyte;
                    }
                    sub_off=string[entry]; //get the substring pointer table offset
                    //pointer table offset + (index x pointer size) + header size
                    ptr_off=sub_off+index*format[str_entry][2]+headersize;
                    //DEBUG:cout<<hex<<(int)format[str_entry][0]<<" "<<(int)format[str_entry][1]<<" "<<(int)format[str_entry][2]<<" "<<(int)format[str_entry][3]<<"\n";
                    //DEBUG:cout<<hex<<"Entry: "<<index<<"Sub: "<<sub_off<<"Ptr: "<<ptr_off<<"\n";
                    fseek(rom,ptr_off,SEEK_SET);
                    readbyte=fgetc(rom);
                    sub_off=readbyte;
                    readbyte=fgetc(rom);
                    sub_off=sub_off+(readbyte*0x100);
                    //determine ROM position
                    if(format[str_entry][2]==3)      //3 byte pointer table
                    {
                        readbyte=fgetc(rom);
                        sub_off=sub_off+(readbyte*0x10000);
                        if(format[str_entry][1]==0)    //LoROM
                            sub_off=(sub_off%0x8000)+(sub_off&0xFF7F0000>>16)*0x8000 + headersize;
                        else
                            sub_off=(sub_off&0xFF7FFFFF) + headersize;
                    }
                    else                            //2 byte pointer. Need to get the bank
                    {
                        if(format[str_entry][1]==0) //LoROM
                            sub_off=sub_off%0x8000+(ptr_off&0xFF7F8000) + headersize;
                        else                        //determine HiROM
                            sub_off=sub_off%0x10000+(ptr_off&0xFF7F0000) + headersize;
                    }
                    fseek(rom,sub_off,SEEK_SET);
                    readbyte=fgetc(rom);
                    entry=readbyte;
                    tbl_off=font[entry];//check if the byte = EOL
                    while(tbl_off!=0xFFFD)
                    {
                        fseek(table,tbl_off,SEEK_SET);
                        check_EOF=fgetc(table);
                        fseek(table,tbl_off,SEEK_SET);
                        readbyte=fgetc(table);
                        while(readbyte!=0x0D && readbyte!=0x0A && check_EOF!=EOF)
                        {
                            tbl_off++;
                            fseek(tdump,write_off,SEEK_SET);
                            fputc(readbyte,tdump);
                            write_off++;
                            fseek(table,tbl_off,SEEK_SET);
                            check_EOF=fgetc(table);
                            fseek(table,tbl_off,SEEK_SET);
                            readbyte=fgetc(table);
                        }
                        sub_off++;
                        fseek(rom,sub_off,SEEK_SET);
                        readbyte=fgetc(rom);
                        entry=readbyte;
                        tbl_off=font[entry];
                    }        
                }//End SNES pointer format
                //MORE FORMAT USAGE DEFINITONS GO HERE.
                //use if(format[0]==FORMAT_ID)
                //ENTRY is the last byte read from the ROM.
                //tbl_off should be FFFC, which signifies substring input.
                else
                    cout<<"Format "<<hex<<(int)format[entry][0]<<" is undefined.\n";
            }//end substring format
            else if(tbl_off==0xFFFA)
            {
                //cout<<"Table switch encountered.\n";
                //cout<<tablefile;
                //cout<<hex<<(int)format[entry][0]<<" "<<(int)format[entry][1];
                x=0;
                while(tablefile[x]!='.')
                    x++;
                x--;
                tablefile[x]=(char)(format[entry][1]+0x30);
                font_loaded=format[entry][1];
                //cout<<tablefile;
                fclose(table);
                if((table=fopen(tablefile,"rb"))==NULL)
                {
                    cout<<"Can't open the table file \""<<tablefile<<"\".\n";
                    system("PAUSE");
                    return EXIT_FAILURE;
                }
                if(load_font_table(font,string,format)==EXIT_FAILURE)
                    return EXIT_FAILURE;
                //system("PAUSE");
            }
            else    
                system("PAUSE");
        }
        else if(tbl_off==0)
        {
            fseek(tdump,write_off,SEEK_SET);
            fputc('(',tdump);
            fprintf(tdump,"%02X",entry);
            write_off+=3;
            fseek(tdump,write_off,SEEK_SET);
            fputc(')',tdump);
            write_off++;
        }
        else
        {
            fseek(table,tbl_off,SEEK_SET);
            check_EOF=fgetc(table);
            fseek(table,tbl_off,SEEK_SET);
            readbyte=fgetc(table);
            while(readbyte!=0x0A && readbyte!=0x0D && check_EOF!=EOF)
            {
                tbl_off++;
                fseek(tdump,write_off,SEEK_SET);
                fputc(readbyte,tdump);
                write_off++;
                fseek(table,tbl_off,SEEK_SET);
                check_EOF=fgetc(table);
                fseek(table,tbl_off,SEEK_SET);
                readbyte=fgetc(table);
            }
        }
        rom_off++;
    }
    cout<<"END PROGRAM.\n";
    system("PAUSE");
    return EXIT_SUCCESS;
}

int hex_return(char readbyte)
{
    if(readbyte>='0' && readbyte<='9')
        return readbyte-'0';
    else if(readbyte>='A' && readbyte <='F')
        return (readbyte-'A')+10;
    else if(readbyte>='a' && readbyte <='f')
        return (readbyte-'a')+10;
    else
    {
        cout<<"HEX VALUE ERROR:"<<hex<<(int)readbyte<<"\n";
        return 0;
    }
}

int load_font_table(unsigned int font[], unsigned long int string[], unsigned char format[][4])
{
    unsigned long int table_off=0;  //offset in font table
    unsigned long int hex_int=0;    //int of a series of hex digits
    unsigned int check_EOF=0;       //int, used to check EOF
    unsigned int pointer=0;         //pointer to entry in font table;
    unsigned char readbyte;         //temp. char, used from reading a byte
    unsigned char quit_entry=0;
    unsigned long int sub_pos;      //substring table position
    int x;
    unsigned char form_str[20];     //hold text string specifying format
    FILE *tbl;
    if ((tbl=fopen(tablefile,"rb"))==NULL)
    {
        cout<<"Couldn't read table file.\n";
        system("PAUSE");
        return EXIT_FAILURE;
    }
    fseek(tbl,table_off,SEEK_SET);
    check_EOF=fgetc(tbl);
    while(check_EOF!=EOF)
    {
        hex_int=0;
        quit_entry=0;
        pointer=0;
        fseek(tbl,table_off,SEEK_SET);
        readbyte=fgetc(tbl);
        table_off++;
        if(readbyte=='*')
        {
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            if(readbyte=='=')
            {
                table_off++;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
            }
            hex_int=hex_return(readbyte);
            pointer=hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            hex_int=hex_return(readbyte);
            pointer=(pointer*16)+hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            if(readbyte=='=')
                table_off++;
            font[pointer]=0xFFFF;
        }
        else if(readbyte=='\\')
        {
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            if(readbyte=='=')
            {
                table_off++;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
            }
            hex_int=hex_return(readbyte);
            pointer=hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            hex_int=hex_return(readbyte);
            pointer=(pointer*16)+hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            font[pointer]=0xFFFE;
        }
        else if(readbyte=='/')
        {
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            if(readbyte=='=')
            {
                table_off++;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
            }
            hex_int=hex_return(readbyte);
            pointer=hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            hex_int=hex_return(readbyte);
            pointer=(pointer*16)+hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            font[pointer]=0xFFFD;
        }
        else if(readbyte=='$')  //substring entry call.
        {
            fseek(tbl,table_off,SEEK_SET);  //check wither TBL had $= or just $
            readbyte=fgetc(tbl);
            if(readbyte=='=')
            {
                table_off++;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
            }
            hex_int=hex_return(readbyte);  //get 10s digit of hex value
            pointer=hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET); //get ones digit of hex value
            readbyte=fgetc(tbl);
            hex_int=hex_return(readbyte);
            pointer=(pointer*16)+hex_int;
            table_off++;         
            //now, we process the table string
            table_off++;          //we expect = sign following call value
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            x=0;
            while(readbyte!=',')  //define format
            {
                form_str[x++]=readbyte;
                table_off++;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
            }
            form_str[x]='\0';    //terminate format string
            if(check_str(form_str,"FIXED\0")==1)  //Found fixed-length string
            {
                font[pointer]=0xFFFC;  //reserved offset for pointer call
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                while(readbyte!=',')       //Find substring table offset
                {
                    sub_pos=sub_pos*16;    //not end, we move one digit over
                    hex_int=hex_return(readbyte);
                    sub_pos=sub_pos+hex_int;
                    table_off++;
                    fseek(tbl,table_off,SEEK_SET);
                    readbyte=fgetc(tbl);
                }
                string[pointer]=sub_pos;
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                check_EOF=fgetc(tbl);
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                while(readbyte!=',')     //Checking for length of fixed strings.
                {
                    sub_pos=sub_pos*16;
                    hex_int=hex_return(readbyte);
                    sub_pos=sub_pos+hex_int;
                    table_off++;
                    fseek(tbl,table_off,SEEK_SET);
                    check_EOF=fgetc(tbl);
                    fseek(tbl,table_off,SEEK_SET);
                    readbyte=fgetc(tbl);
                }
                format[pointer][0]=0x0;
                format[pointer][1]=(char)sub_pos;
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                check_EOF=fgetc(tbl);
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                while(readbyte!=0x0A && readbyte!=0x0D && check_EOF!=EOF)
                //checking for size of subtring entry (only 1 byte long entry
                //IDs (max 256 entries) supported currently)
                {
                    sub_pos=sub_pos*16;
                    hex_int=hex_return(readbyte);
                    sub_pos=sub_pos+hex_int;
                    table_off++;
                    fseek(tbl,table_off,SEEK_SET);
                    check_EOF=fgetc(tbl);
                    fseek(tbl,table_off,SEEK_SET);
                    readbyte=fgetc(tbl);
                }
                format[pointer][2]=(char)sub_pos;
                table_off+=2;                
            }//end of definition for FIXED format
            else if(check_str(form_str,"TABLE\0")==1)
            {
                font[pointer]=0xFFFA;
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                //if it is a number
                {
                    while(readbyte!=0x0A && readbyte!=0x0D && check_EOF!=EOF)
                    //checking for table number
                    {
                        sub_pos=sub_pos*16;
                        hex_int=hex_return(readbyte);
                        sub_pos=sub_pos+hex_int;
                        table_off++;
                        fseek(tbl,table_off,SEEK_SET);
                        check_EOF=fgetc(tbl);
                        fseek(tbl,table_off,SEEK_SET);
                        readbyte=fgetc(tbl);
                    }//end find table number
                    format[pointer][0]=(char)1;            //= fixed table number
                    format[pointer][1]=(char)sub_pos;      //table number
                }//end if it is a number
            }//end of definition for TABLE swap
            else if(check_str(form_str,"SNES\0")==1)
            {
                font[pointer]=0xFFFC;  //reserved offset for pointer call
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                if(readbyte=='L')
                   format[pointer][1]=0;
                else
                   format[pointer][1]=1;
                table_off+=2;
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                while(readbyte!=',')       //Find substring table offset
                {
                    sub_pos=sub_pos*16;    //not end, we move one digit over
                    hex_int=hex_return(readbyte);
                    sub_pos=sub_pos+hex_int;
                    table_off++;
                    fseek(tbl,table_off,SEEK_SET);
                    readbyte=fgetc(tbl);
                }
                string[pointer]=sub_pos;
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                check_EOF=fgetc(tbl);
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                while(readbyte!=',')     //Checking for byte size 2-byte or 3-byte
                {
                    sub_pos=sub_pos*16;
                    hex_int=hex_return(readbyte);
                    sub_pos=sub_pos+hex_int;
                    table_off++;
                    fseek(tbl,table_off,SEEK_SET);
                    check_EOF=fgetc(tbl);
                    fseek(tbl,table_off,SEEK_SET);
                    readbyte=fgetc(tbl);
                }
                format[pointer][0]=0x2; //=SNES pointer
                format[pointer][2]=(char)sub_pos;
                table_off++;
                sub_pos=0;
                fseek(tbl,table_off,SEEK_SET);
                check_EOF=fgetc(tbl);
                fseek(tbl,table_off,SEEK_SET);
                readbyte=fgetc(tbl);
                while(readbyte!=0x0A && readbyte!=0x0D && check_EOF!=EOF)
                //checking for size of subtring entry (only 1 byte long entry
                //IDs (max 256 entries) supported currently)
                {
                    sub_pos=sub_pos*16;
                    hex_int=hex_return(readbyte);
                    sub_pos=sub_pos+hex_int;
                    table_off++;
                    fseek(tbl,table_off,SEEK_SET);
                    check_EOF=fgetc(tbl);
                    fseek(tbl,table_off,SEEK_SET);
                    readbyte=fgetc(tbl);
                }
                format[pointer][3]=(char)sub_pos;
                table_off+=2;
            }//end of definition for SNES
            //MORE FOMRAT IDENTIFIES GO HERE.
            //begin with statement if(check_str(form_str,"FORMAT_NAME\0")==1)
            //Put the format ID into
            //format[0].
            //Use format[0]-[20] for whatever variables are needed.
        }
        else
        {
            hex_int=hex_return(readbyte);
            pointer=hex_int;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            hex_int=hex_return(readbyte);
            pointer=(pointer*16)+hex_int;
            table_off++;
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            if(readbyte=='=')
                table_off++;
            font[pointer]=table_off;
            table_off++;
        }
        fseek(tbl,table_off,SEEK_SET);
        check_EOF=fgetc(tbl);
        while(check_EOF!=EOF && quit_entry==0)   //check for table entry end
        {
            fseek(tbl,table_off,SEEK_SET);
            readbyte=fgetc(tbl);
            if(readbyte==0x0A || readbyte==0x0D)
            {
                quit_entry=1;
                table_off+=2;
                fseek(tbl,table_off,SEEK_SET);
                check_EOF=fgetc(tbl);
            }
            else
            {
                table_off++;
                fseek(tbl,table_off,SEEK_SET);
                check_EOF=fgetc(tbl);
            }//end check for EOL or quit
        }//end else - not control code
    }//not end of table
    fclose(tbl);
}

int check_str(unsigned char string1[], const char string2[])
{
    int index=0;
    while(string1[index]!='\0')
    {
        if(string1[index]!=string2[index])
           return 0;
        index++;
    } 
    return 1;
}

int hex_to_int(unsigned char string[])
{
    int x=0;
    unsigned long int intvalue=0;
    int hex_int;
    while(string[x]!='\0')
    {
        intvalue=intvalue*16;
        hex_int=hex_return(string[x]);
        intvalue=intvalue+hex_int;
        x++;
    }
    return intvalue;
}

int power(int x, int y)
{
    int z=x;
    while(y>0)
    {
        z=z*x;
        y--;
    }
    return z;
}
